package com.itcast.bulls.stock.deal.service;

import com.itcast.bulls.stock.common.exception.ComponentException;
import com.itcast.bulls.stock.common.exception.constants.ApplicationErrorCodeEnum;
import com.itcast.bulls.stock.common.utils.GlobalConstants;
import com.itcast.bulls.stock.deal.dao.trade.ITradeOrderDao;
import com.itcast.bulls.stock.deal.service.trade.IStockDealService;
import com.itcast.bulls.stock.entity.quote.TradeStockQuoteLast;
import com.itcast.bulls.stock.entity.trade.TradeOrder;
import com.itcast.bulls.stock.struct.grpc.trade.PlaceOrderReponse;
import com.itcast.bulls.stock.struct.grpc.trade.PlaceOrderRequest;
import com.itcast.bulls.stock.struct.grpc.trade.StockTradeDealServiceGrpc;
import io.grpc.stub.StreamObserver;
import lombok.extern.log4j.Log4j2;
import net.devh.boot.grpc.server.service.GrpcService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.Cache;
import org.springframework.data.redis.cache.RedisCacheManager;
import org.springframework.transaction.annotation.Transactional;

/**
 * <p>Description: </p>
 * @date
 * @author
 * @version 1.0
 * <p>Copyright:Copyright(c)2020</p>
 */
@GrpcService
@Log4j2
public class GrpcStockDealService extends StockTradeDealServiceGrpc.StockTradeDealServiceImplBase {

    @Autowired
    private ITradeOrderDao tradeOrderDao;
    /**
     * 缓存管理器
     */
    @Autowired
    private RedisCacheManager redisCacheManager;

    /**
     * 交易服务业务层接口
     */
    @Autowired
    private IStockDealService stockDealService;

    /**
     * <pre>
     * place order
     * </pre>
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void placeOrder(PlaceOrderRequest request,
                           StreamObserver<PlaceOrderReponse> responseObserver) {
        log.info("Process in GrpcStockDealService.placeOrder, request: " + request);
        PlaceOrderReponse reponse = null;

        try {
            // 1. 获取实时行情，取得对应股票的实时报价
            Cache lastPriceCache = redisCacheManager.getCache(GlobalConstants.STOCK_QUOTE_LAST_PRICE_KEY);
            Cache.ValueWrapper quoteLastWrapper = lastPriceCache.get(request.getStockId());
            if(quoteLastWrapper == null ) {
                throw new ComponentException(ApplicationErrorCodeEnum.QUOTE_LAST_PRICE_NOT_FOUND);
            }
            // 2. 根据下单类型判断(两种类型：一种市价委托，另一种是限价委托)，调用相应业务逻辑处理接口
            TradeStockQuoteLast quoteLast = (TradeStockQuoteLast) quoteLastWrapper.get();
            TradeOrder tradeOrder = null;
            // 2. 根据下单类型进行处理： 第一种是市价委托下单
            if(request.getType() == GlobalConstants.ORDER_TYPE_MARKET) {
                // 调用交易服务的业务层接口，处理市价下单
                tradeOrder = stockDealService.processMarketOrder(request, quoteLast);
                reponse = PlaceOrderReponse.newBuilder().setStatus(GlobalConstants.STATUS_OK).setDealId(tradeOrder.getId()).setMessage("下单成功！").build();

            }
            // 3. 根据下单类型进行处理： 第二种是限价委托下单
            else if(request.getType() == GlobalConstants.ORDER_TYPE_LIMIT) {
                // TODO 实现限价委托下单功能时在做具体处理
            }

        }catch(ComponentException e) {
            // 业务异常返回
            log.error(e.getMessage(), e);
            reponse = PlaceOrderReponse.newBuilder().setStatus(e.getErrorCodeEnum().getCode()).setMessage(e.getErrorCodeEnum().getMessage()).build();
        }catch(Exception e) {
            // 系统异常信息返回
            log.error(e.getMessage(), e);
            reponse = PlaceOrderReponse.newBuilder().setStatus(GlobalConstants.STATUS_ERROR).setMessage(e.getMessage() == null ? "null":e.getMessage()).build();
        }

        responseObserver.onNext(reponse);
        responseObserver.onCompleted();

    }

}
